//---------------------------------------------------------------------
//---------------------------------------------------------------------
#include <string.h>
#include <picogui.h>

#include <vl_base.h>
#include <vl_file.h>

//---------------------------------------------------------------------
#define MAX_BUFFER_SIZE				64
#define MAX_IMG_HEADER_SIZE			MAX_BUFFER_SIZE

//---------------------------------------------------------------------
int get_ipl_buffer( IPL_BUFHANDLE img, IPL_LONG *w, IPL_LONG *h, IPL_LONG *bps )
{
	int ret = -1;

	if( img != IPL_DONT_CARE )
	{
		if( IPL_BufInquire( img, IPL_XSIZE, w ))
		{
			if( IPL_BufInquire( img, IPL_YSIZE, h ))
			{
				if( IPL_BufInquire( img, IPL_NB_BANDS, bps ))
					ret = 0;
				else
					fprintf( stderr, "*** get_ipl_img - inquire the nb. of bands failed!!\n" );
			}
			else
				fprintf( stderr, "*** get_ipl_img - inquire the image height failed!!\n" );
		}
		else
			fprintf( stderr, "*** get_ipl_img - inquire the image width failed!!\n" );
	}
	else
		fprintf( stderr, "*** get_ipl_img - wrong image data!!\n" );

	return ret;
}

//---------------------------------------------------------------------
int read_data_from_ipl_buffer( IPL_BUFHANDLE img, IPL_LONG w, IPL_LONG h, IPL_LONG bps, unsigned char *DataArray )
{
	int ret = 0;

	switch( bps )
	{
	case 1:
		if( !IPL_GetDispData( img, IPL_DONT_CARE, 0,0,w,h, 0,0, IPL_MONO,IPL_NOT_INVERTED,
						0,0,0, 0,0,0, 0,0,0, 0, DataArray ))
			ret = -1;
		break;

	case 3:
		if( !IPL_GetDispData( img, IPL_DONT_CARE, 0,0,w,h, 0,0, IPL_RGB,IPL_NOT_INVERTED,
						2,1,0, 0,0,0, 1,1,1, 0, DataArray ))
			ret = -1;
		break;

	default:
		fprintf( stderr, "*** read_data_from_ipl_buffer - wrong bytes per sample number (%ld)!!", bps );
		ret = -1;
		break;
	}

	return ret;
}

//---------------------------------------------------------------------
int make_pg_header( IPL_LONG width, IPL_LONG height, IPL_LONG bps, char *iheader )
{
	int len = -1;

	switch( bps )
	{
	case 1:
		len = sprintf( iheader, "P5\n# ipl Gray test header\n%ld %ld\n255\n", width, height );
		break;

	case 3:
		len = sprintf( iheader, "P6\n# ipl RGB test header\n%ld %ld\n255\n", width, height );
		break;

	default:
		fprintf( stderr, "*** make_pg_header - wrong bytes per sample number (%ld)!!", bps );
		break;
	}

	return len;
}

//---------------------------------------------------------------------
int set_pg_img_data( struct pgmemdata *pg_data, unsigned char *DataArray, int isize, char *iheader, int hlen )
{
	int ret = -1;
	unsigned char *pimg = NULL;

	if(( pimg = (unsigned char *) malloc(( isize + hlen ) * sizeof(unsigned char))) != NULL )
	{
		memset( pimg, 0, isize + hlen );
		memcpy( pimg + 0, iheader, hlen );
		memcpy( pimg + hlen, DataArray, isize );

		pg_data->pointer = (void*)pimg;
		pg_data->size = isize + hlen;
		pg_data->flags = PGMEMDAT_NEED_FREE;
		ret = 0;
	}
	else
		fprintf( stderr,"*** set_pg_img_data - out of memory!!\n" );

	return ret;
}

//---------------------------------------------------------------------
pghandle pgNewIPLBitmap( IPL_BUFHANDLE image )
{
	pghandle img_h = 0;
	int len = -1;
	IPL_LONG width = 0, height = 0, bps = 0;
	unsigned char *DataArray = NULL;
	char iheader[MAX_IMG_HEADER_SIZE];
	struct pgmemdata pg_data = { NULL, 0, 0 };

	memset( iheader, 0, MAX_IMG_HEADER_SIZE );

	if( image != IPL_DONT_CARE )
	{
		if( get_ipl_buffer( image, &width, &height, &bps ) == 0 )
		{
			if(( DataArray = (unsigned char *) malloc(( bps * width * height ) * sizeof(unsigned char))) != NULL )
			{
				if( read_data_from_ipl_buffer( image, width, height, bps, DataArray ) == 0 )
				{
					if(( len = make_pg_header( width, height, bps, iheader )) > 0 )
					{
						if( set_pg_img_data( &pg_data, DataArray, width * height * bps, iheader, len ) == 0 )
							img_h = pgNewBitmap( pg_data );
					}
				}
			}
			else
				fprintf( stderr,"*** out of memory!!\n" );

			if( DataArray != NULL )
			{
				free( DataArray );
				DataArray = NULL;
			}
		}
	}
	else
		fprintf( stderr,"*** no image data found!!\n" );


	return img_h;
}

//---------------------------------------------------------------------
int btnExit( struct pgEvent * /*evt*/ )
{
	pgExitEventLoop( );
	return 0;
}

//---------------------------------------------------------------------
int main( int argc, char **argv )
{
	IPL_BUFHANDLE image = IPL_DONT_CARE;
	const char *filename = "brain.bmp";
	pghandle box=0, panel=0, bitmapwidget=0;
	int w = 0, h = 0;
	pghandle bitmap;
	char infos[MAX_BUFFER_SIZE];
	memset( infos, 0, MAX_BUFFER_SIZE );

	pgInit( argc, argv );

	panel = pgRegisterApp( PG_APP_NORMAL, "mvIMPACT Image Viewer", 0 );

	box = pgNewWidget( PG_WIDGET_SCROLLBOX, PG_DERIVE_INSIDE, panel );
	pgSetWidget( PGDEFAULT, PG_WP_THOBJ, pgFindThemeObject("imgview.box"), 0 );

	pgNewWidget( PG_WIDGET_BUTTON, PG_DERIVE_INSIDE, panel );
	pgSetWidget( PGDEFAULT, PG_WP_SIDE, PG_S_TOP,
					PG_WP_TEXT, pgNewString("Klick here to terminate the program!"),
					PG_WP_FONT, pgNewFont( NULL, 12, PG_FSTYLE_BOLD ),
					PG_WP_BITMAP, pgNewBitmap( pgFromFile("tux.pnm")), 0 );

	pgBind( PGDEFAULT, PG_WE_ACTIVATE, &btnExit, NULL );

	bitmapwidget = pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, box );
	pgSetWidget( PGDEFAULT, PG_WP_SIDE, PG_S_ALL, PG_WP_MARGIN, 0, 0 );

	pgDelete( pgGetWidget( bitmapwidget, PG_WP_BITMAP ));


	if( argc > 1 )
		filename = argv[1];

	if( IPL_FlImport( &image, NULL, FL_DONT_CARE, filename ))
	{
		bitmap = pgNewIPLBitmap( image );

		pgcontext gc = pgNewBitmapContext( bitmap );
		pgSetColor( gc, 0xFF0002 );
		pgLine( gc, 10, 10, 100, 10 );
		pgLine( gc, 100, 10, 100, 100 );
		pgLine( gc, 100, 100, 10, 100 );
		pgLine( gc, 10, 100, 10, 10 );

		pgSetColor( gc, 0xFFFFFF );
		pgText( gc, 15,45, pgNewString("Hello, world! :o)"));

		pgSizeBitmap( &w, &h, bitmap );
		sprintf( infos,"%s - %dx%d Image", filename, w, h );

		pgNewWidget( PG_WIDGET_LABEL, PG_DERIVE_INSIDE, box );
		pgSetWidget( PGDEFAULT, PG_WP_SIDE, PG_S_BOTTOM, PG_WP_TEXT, pgNewString( infos ),
					PG_WP_FONT, pgNewFont( NULL, 8, PG_FSTYLE_DEFAULT ), 0 );

		pgSetWidget( bitmapwidget, PG_WP_BITMAP, bitmap, 0 );
	}
	else
	{
		fprintf( stderr,"*** the file >>%s<< not loaded!!\n", filename );
		return -1;
	}

	pgEventLoop( );

	return 0;
}

//---------------------------------------------------------------------
//---------------------------------------------------------------------
